#!/usr/bin/env python

"""Tests for `pysesame3` package."""

import pytest

from pysesame3.auth import WebAPIAuth


def test_WebAPIAuth_raises_exception_on_missing_arguments():
    with pytest.raises(TypeError):
        WebAPIAuth()


def test_WebAPIAuth_raises_exception_on_invalid_apikey():
    with pytest.raises(ValueError) as excinfo:
        WebAPIAuth(apikey="thisisfake")
    assert "length should be 40" in str(excinfo.value)



import requests


class WebAPIAuth(requests.auth.AuthBase):
    def __init__(self, apikey: str):
        """Generic Implementation for Web API Authentication.

        Args:
            apikey (str): API Key
        """
        self._apikey = apikey

        if len(self._apikey) != 40:
            raise ValueError("Invalid API Key - length should be 40.")

    def __call__(self, r):
        r.headers["x-api-key"] = self._apikey
        return r



from __future__ import annotations

from enum import Enum, IntEnum, auto
from typing import TYPE_CHECKING

from pysesame3.cloud import SesameCloud
from pysesame3.device import SesameLocker

if TYPE_CHECKING:
    from pysesame3.auth import WebAPIAuth
    from pysesame3.helper import CHSesame2MechStatus


class CHSesame2CMD(IntEnum):
    LOCK = 82
    UNLOCK = 83
    TOGGLE = 88


class CHSesame2ShadowStatus(Enum):
    LockedWm = auto()
    UnlockedWm = auto()
    MovedWm = auto()


class CHSesame2(SesameLocker):
    """ """

    def __init__(
        self, authenticator: WebAPIAuth, device_uuid: str, secret_key: str
    ) -> None:
        """SESAME3 Device Specific Implementation.

        Args:
            authenticator (WebAPIAuth):
            device_uuid (str): The UUID of the device
            secret_key (str): The secret key of the device
        """
        super().__init__(authenticator)

        self.setDeviceUUID(device_uuid)
        self.setSecretKey(secret_key)

        if self.mechStatus.isInLockRange():
            self._deviceShadowStatus = CHSesame2ShadowStatus.LockedWm
        else:
            self._deviceShadowStatus = CHSesame2ShadowStatus.UnlockedWm

    @property
    def mechStatus(self) -> CHSesame2MechStatus:
        """Returns a mechanical status of a device.

        Returns:
            CHSesame2MechStatus: Current mechanical status of the device.
        """
        return SesameCloud(self).getMechStatus()

    def getDeviceShadowStatus(self) -> CHSesame2ShadowStatus:
        """Returns a shadow status of a device.

        Returns:
            CHSesame2ShadowStatus: Shadow (assumed) status of the device.
        """
        return self._deviceShadowStatus

    def setDeviceShadowStatus(self, status: CHSesame2ShadowStatus) -> None:
        """Sets a shadow status of a device.

        Args:
            status (CHSesame2ShadowStatus): Desired shadow (assumed) status

        Raises:
            ValueError: If `status` is invalid.
        """
        if not isinstance(status, CHSesame2ShadowStatus):
            raise ValueError("Invalid CHSesame2ShadowStatus")
        self._deviceShadowStatus = status

    def lock(self) -> bool:
        """Locking.

        Returns:
            bool: `True` if it is successfully locked, `False` if not.
        """
        result = SesameCloud(self).sendCmd(CHSesame2CMD.LOCK)
        if result:
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.LockedWm)
        return result

    def unlock(self) -> bool:
        """Unlocking.

        Returns:
            bool: `True` if it is successfully unlocked, `False` if not.
        """
        result = SesameCloud(self).sendCmd(CHSesame2CMD.UNLOCK)
        if result:
            self.setDeviceShadowStatus(CHSesame2ShadowStatus.UnlockedWm)
        return result

    def toggle(self) -> bool:
        """Toggle.

        Returns:
            bool: `True` if it is successfully toggled, `False` if not.
        """
        if self.getDeviceShadowStatus() == CHSesame2ShadowStatus.LockedWm:
            return self.unlock()
        elif self.getDeviceShadowStatus() == CHSesame2ShadowStatus.UnlockedWm:
            return self.lock()

    def __str__(self) -> str:
        """Return a string representation of an object.

        Returns:
            str: The string representation of the object.
        """
        return f"CHSesame2(deviceUUID={self.getDeviceUUID()}, deviceModel={self.productModel}, sesame2PublicKey={self.getSesame2PublicKey()}, mechStatus={self.mechStatus})"



#!/usr/bin/env python

"""Tests for `pysesame3` package."""

import pytest
import requests_mock
from pysesame3.cloud import SesameCloud
from pysesame3.history import CHSesame2History

from .utils import load_fixture


@pytest.fixture(autouse=True)
def mock_requests():
    with requests_mock.Mocker() as mock:
        mock.get(
            "https://app.candyhouse.co/api/sesame2/FAKEUUID/history?page=0&lg=10",
            json=load_fixture("history.json"),
        )

        yield mock


class TestSesameCloudBroken:
    def test_SesameCloud_raises_exception_on_authenticator_missing(self):
        with pytest.raises(TypeError) as excinfo:
            SesameCloud()
        assert "missing 1 required positional argument" in str(excinfo.value)


class TestSesameCloudOfficial:
    @pytest.fixture(autouse=True)
    def _load_SesameCloud(self):
        class FakeDevice:
            @property
            def authenticator(self):
                return None

            def getDeviceUUID(self):
                return "FAKEUUID"

        self.cl = SesameCloud(FakeDevice())

    def test_SesameCloud_getHistoryEntries(self):
        entries = self.cl.getHistoryEntries()

        assert isinstance(entries, list)
        assert len(entries) == 2
        assert isinstance(entries[0], CHSesame2History)
        assert isinstance(entries[1], CHSesame2History)
